/**
	$Id: Slot.java 5447 2010-10-19 11:02:17Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent.slotmanager;

import net.sf.basedb.core.Job;

/**
	Represents a slot that has been assigned to a job.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-10-19 13:02:17 +0200 (ti, 19 okt 2010) $
*/
public class Slot
{
	
	private final String id;
	private final Job.ExecutionTime estimated;
	
	/**
		Create a new slot and automatically assign an id to
		it.
		@param estimated The estimated execution time for the job
	*/
	public Slot(Job.ExecutionTime estimated)
	{
		this.id = Integer.toString(hashCode());
		this.estimated = estimated;
	}
	
	/**
		Create a new slot.
		@param id The id of the slot
		@param estimated The estimated execution time for the job
	*/
	public Slot(String id, Job.ExecutionTime estimated)
	{
		this.id = id;
		this.estimated = estimated;
	}
	
	/**
		Get the id of the slot.
	*/
	public String getId()
	{
		return id;
	}
	
	/**
		Get the estimated execution time for this slot.
	*/
	public Job.ExecutionTime getEstimatedExecutionTime()
	{
		return estimated;
	}
	
	@Override
	public String toString()
	{
		return "Slot[id=" + id + ";" + estimated.name() + "]";
	}
	
}
