/**
	$Id: DebugOnSaveActionFactory.java 5501 2010-11-18 11:05:43Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.edit;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	On-save action factory that create actions that print a log message
	to System.out for all events. Useful for debugging and testing.

	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-18 12:05:43 +0100 (to, 18 nov 2010) $
*/
public class DebugOnSaveActionFactory
	extends AbstractJspActionFactory<OnSaveAction>

{

	public DebugOnSaveActionFactory()
	{}

	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public OnSaveAction[] getActions(InvokationContext<? super OnSaveAction> context)
	{
		return new OnSaveAction[] { new DebugOnSaveAction((JspContext)context.getClientContext()) };
	}
	// --------------------------------
	
	public static class DebugOnSaveAction
		extends AbstractOnSaveAction
	{
		public DebugOnSaveAction(JspContext jspContext)
		{
			super(jspContext);
		}

		@Override
		public void onSave()
		{
			System.out.println("onSave: " + getJspContext().getCurrentItem());
		}

		@Override
		public void onCommit()
		{
			System.out.println("onCommit: " + getJspContext().getCurrentItem());
		}

		@Override
		public void onRollback(Exception ex)
		{
			System.out.println("onRollback: " + getJspContext().getCurrentItem());
			ex.printStackTrace(System.out);
		}
	}
	
}
