/**
	$Id: FixedTabFactory.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.tabcontrol;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	A simple implementation of a tab action factory that 
	creates the same fixed tab for all users not using any 
	context information.
	<p>
	If the extension point supports scripts and stylesheets, use the
	{@link #setScript(String)} and {@link #setStylesheet(String)} methods.
	Each call to these methods will add the argument to a set. Use
	the {@link #getScripts()} and {@link #getStylesheets()} methods to gain
	access to the sets, for example, to remove added items.
	
	@author nicklas
	@since 2.17
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (må, 20 apr 2015) $
*/
public class FixedTabFactory 
	extends AbstractJspActionFactory<TabAction>
	implements TabAction 
	
{

	private TabAction[] actions;
	
	private String id;
	private String clazz;
	private String style;
	private String title;
	private String tooltip;
	private String helpId;
	private boolean visible;
	private String content;
	private int tabIndex;
	
	/**
		Create a new factory instance.
	*/
	public FixedTabFactory()
	{
		actions = new TabAction[] { this };
		visible = true;
	}
	
	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public TabAction[] getActions(InvokationContext<? super TabAction> context) 
	{
		return actions;
	}
	// ------------------------------------

	/*
		From the TabAction interface
		--------------------------------
	*/
	@Override
	public String getId() 
	{
		return id;
	}

	@Override
	public String getClazz() 
	{
		return clazz;
	}

	@Override
	public String getStyle() 
	{
		return style;
	}

	@Override
	public String getTitle() 
	{
		return title;
	}

	@Override
	public String getTooltip() 
	{
		return tooltip;
	}

	@Override
	public boolean isVisible() 
	{
		return visible;
	}
	
	@Override
	public String getHelpId()
	{
		return helpId;
	}

	@Override
	public String getContent()
	{
		return content;
	}
	
	@Override
	public int getTabIndex()
	{
		return tabIndex;
	}
	// ------------------------------------

	public void setId(String id) 
	{
		this.id = id;
	}

	public void setClazz(String clazz) 
	{
		this.clazz = clazz;
	}

	public void setStyle(String style) 
	{
		this.style = style;
	}

	public void setTitle(String title) 
	{
		this.title = title;
	}

	public void setTooltip(String tooltip) 
	{
		this.tooltip = tooltip;
	}

	public void setVisible(boolean visible) 
	{
		this.visible = visible;
	}

	public void setVisible(String visible) 
	{
		this.visible = Values.getBoolean(visible);
	}

	public void setHelpId(String helpId)
	{
		this.helpId = helpId;
	}

	public void setContent(String content)
	{
		this.content = content;
	}
	
	public void setTabIndex(String tabIndex)
	{
		this.tabIndex = Values.getInt(tabIndex);
	}

}
