/*
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.core.data.StringParameterValueData;

/**
	This class represent a parameter type that is a password 
	(or any other type of masked string). Max length can not
	be longer than 255, and multiple values are not supported.

	@author Nicklas
	@since 3.7
*/
public class PasswordParameterType
	extends StringParameterType
{

	/**
		Create a new password parameter type, setting the max length 
		to 255 and nulls are allowed.
	*/
	public PasswordParameterType()
	{
		super();
	}

	/**
		Create a new password parameter type.
		@param maxLength The maximum length of the string that is allowed or null
			to allow as long as possible
		@param notNull FALSE if nulls values are allowed, TRUE otherwise
	*/
	public PasswordParameterType(Integer maxLength, boolean notNull)
	{
		super(maxLength == null ? 255 : Math.min(maxLength, 255), null, notNull);
	}
	
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "PasswordParameterType: MaxLength " + getMaxLength();
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Create a new {@link StringParameterValueData} object.
	*/
	@Override
	ParameterValueData<String> newParameterValueData()
	{
		return new StringParameterValueData();
	}

	@Override
	public boolean isMasked() 
	{
		return true;
	}
	// -------------------------------------------

}
