/**
	$Id: QueryWrapper.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZonedDateTime;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Stream;

import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.Parameter;
import javax.persistence.TemporalType;

import org.hibernate.CacheMode;
import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.query.ParameterMetadata;
import org.hibernate.query.Query;
import org.hibernate.query.QueryParameter;
import org.hibernate.query.QueryProducer;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.engine.spi.RowSelection;
import org.hibernate.transform.ResultTransformer;
import org.hibernate.type.Type;

/**
	A simple wrapper around hibernate queries which simply forwards 
	all calls to a parent query.
	
	@author nicklas
	@version 2.5
	@base.modified $Date: 2017-05-22 14:35:27 +0200 (må, 22 maj 2017) $
*/
public class QueryWrapper<R>
	implements Query<R>
{

	protected final Query<R> query;
	
	public QueryWrapper(Query<R> query)
	{
		this.query = query;
	}
	
	@Override
	public int executeUpdate()
		throws HibernateException
	{
		return query.executeUpdate();
	}

	@Override
	@Deprecated
	public String[] getNamedParameters() 
		throws HibernateException
	{
		return query.getNamedParameters();
	}

	@Override
	public String getQueryString()
	{
		return query.getQueryString();
	}

	@Override
	@Deprecated
	public String[] getReturnAliases() 
		throws HibernateException
	{
		return query.getReturnAliases();
	}

	@Override
	@Deprecated
	public Type[] getReturnTypes() 
		throws HibernateException
	{
		return query.getReturnTypes();
	}

	@Override
	@Deprecated
	public boolean isReadOnly()
	{
		return query.isReadOnly();
	}

	@Override
	@Deprecated
	public Iterator<R> iterate() 
		throws HibernateException
	{
		return query.iterate();
	}

	@Override
	public List<R> list() 
		throws HibernateException
	{
		return query.list();
	}

	@Override
	public ScrollableResults scroll() 
		throws HibernateException
	{
		return query.scroll();
	}

	@Override
	public ScrollableResults scroll(ScrollMode scrollMode)
		throws HibernateException
	{
		return query.scroll(scrollMode);
	}

	@Override
	@Deprecated
	public Query<R> setBigDecimal(int position, BigDecimal number)
	{
		query.setBigDecimal(position, number);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBigDecimal(String name, BigDecimal number)
	{
		query.setBigDecimal(name, number);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBigInteger(int position, BigInteger number)
	{
		query.setBigInteger(position, number);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBigInteger(String name, BigInteger number)
	{
		query.setBigInteger(name, number);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBinary(int position, byte[] val)
	{
		query.setBinary(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBinary(String name, byte[] val)
	{
		query.setBinary(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBoolean(int position, boolean val)
	{
		query.setBoolean(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setBoolean(String name, boolean val)
	{
		query.setBoolean(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setByte(int position, byte val)
	{
		query.setByte(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setByte(String name, byte val)
	{
		query.setByte(name, val);
		return this;
	}

	@Override
	public Query<R> setCacheMode(CacheMode cacheMode)
	{
		query.setCacheMode(cacheMode);
		return this;
	}

	@Override
	public Query<R> setCacheRegion(String cacheRegion)
	{
		query.setCacheRegion(cacheRegion);
		return this;
	}

	@Override
	public Query<R> setCacheable(boolean cacheable)
	{
		query.setCacheable(cacheable);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCalendar(int position, Calendar calendar)
	{
		query.setCalendar(position, calendar);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCalendar(String name, Calendar calendar)
	{
		query.setCalendar(name, calendar);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCalendarDate(int position, Calendar calendar)
	{
		query.setCalendarDate(position, calendar);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCalendarDate(String name, Calendar calendar)
	{
		query.setCalendarDate(name, calendar);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCharacter(int position, char val)
	{
		query.setCharacter(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setCharacter(String name, char val)
	{
		query.setCharacter(name, val);
		return this;
	}

	@Override
	public Query<R> setComment(String comment)
	{
		query.setComment(comment);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setDate(int position, Date date)
	{
		query.setDate(position, date);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setDate(String name, Date date)
	{
		query.setDate(name, date);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setDouble(int position, double val)
	{
		query.setDouble(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setDouble(String name, double val)
	{
		query.setDouble(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setEntity(int position, Object val)
	{
		query.setEntity(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setEntity(String name, Object val)
	{
		query.setEntity(name, val);
		return this;
	}

	@Override
	public Query<R> setFetchSize(int fetchSize)
	{
		query.setFetchSize(fetchSize);
		return this;
	}

	@Override
	public Query<R> setFirstResult(int firstResult)
	{
		query.setFirstResult(firstResult);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setFloat(int position, float val)
	{
		query.setFloat(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setFloat(String name, float val)
	{
		query.setFloat(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setFlushMode(FlushMode flushMode)
	{
		query.setFlushMode(flushMode);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setInteger(int position, int val)
	{
		query.setInteger(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setInteger(String name, int val)
	{
		query.setInteger(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setLocale(int position, Locale locale)
	{
		query.setLocale(position, locale);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setLocale(String name, Locale locale)
	{
		query.setLocale(name, locale);
		return this;
	}

	@Override
	public Query<R> setLockMode(String alias, LockMode lockMode)
	{
		query.setLockMode(alias, lockMode);
		return this;
	}
	
	@Override
	public Query<R> setLockOptions(LockOptions lockOptions)
	{
		query.setLockOptions(lockOptions);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setLong(int position, long val)
	{
		query.setLong(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setLong(String name, long val)
	{
		query.setLong(name, val);
		return this;
	}

	@Override
	public Query<R> setMaxResults(int maxResults)
	{
		query.setMaxResults(maxResults);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, Object val)
		throws HibernateException
	{
		query.setParameter(position, val);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, Object val) 
		throws HibernateException
	{
		query.setParameter(name, val);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, Object val, Type type)
	{
		query.setParameter(position, val, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, Object val, Type type)
	{
		query.setParameter(name, val, type);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Query<R> setParameterList(String name, Collection vals)
	{
		query.setParameterList(name, vals);
		return this;
	}

	@Override
	public Query<R> setParameterList(String name, Object[] vals)
	{
		query.setParameterList(name, vals);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Query<R> setParameterList(String name, Collection vals, Type type)
	{
		query.setParameterList(name, vals, type);
		return this;
	}

	@Override
	public Query<R> setParameterList(String name, Object[] vals, Type type)
	{
		query.setParameterList(name, vals, type);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setParameters(Object[] values, Type[] types)
	{
		query.setParameters(values, types);
		return this;
	}

	@Override
	public Query<R> setProperties(Object bean) 
		throws HibernateException
	{
		query.setProperties(bean);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Query<R> setProperties(Map bean) 
		throws HibernateException
	{
		query.setProperties(bean);
		return this;
	}

	@Override
	public Query<R> setReadOnly(boolean readOnly)
	{
		query.setReadOnly(readOnly);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setResultTransformer(ResultTransformer transformer)
	{
		query.setResultTransformer(transformer);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setSerializable(int position, Serializable val)
	{
		query.setSerializable(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setSerializable(String name, Serializable val)
	{
		query.setSerializable(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setShort(int position, short val)
	{
		query.setShort(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setShort(String name, short val)
	{
		query.setShort(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setString(int position, String val)
	{
		query.setString(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setString(String name, String val)
	{
		query.setString(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setText(int position, String val)
	{
		query.setText(position, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setText(String name, String val)
	{
		query.setText(name, val);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setTime(int position, Date date)
	{
		query.setTime(position, date);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setTime(String name, Date date)
	{
		query.setTime(name, date);
		return this;
	}

	@Override
	public Query<R> setTimeout(int timeout)
	{
		query.setTimeout(timeout);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setTimestamp(int position, Date date)
	{
		query.setTimestamp(position, date);
		return this;
	}

	@Override
	@Deprecated
	public Query<R> setTimestamp(String name, Date date)
	{
		query.setTimestamp(name, date);
		return this;
	}

	@Override
	public R uniqueResult() 
		throws HibernateException
	{
		return query.uniqueResult();
	}

	@Override
	@Deprecated
	public FlushModeType getFlushMode() 
	{
		return query.getFlushMode();
	}

	@Override
	public CacheMode getCacheMode() 
	{
		return query.getCacheMode();
	}

	@Override
	@Deprecated
	public boolean isCacheable() 
	{
		return query.isCacheable();
	}

	@Override
	public String getCacheRegion() 
	{
		return query.getCacheRegion();
	}

	@Override
	@Deprecated
	public Integer getTimeout() 
	{
		return query.getTimeout();
	}

	@Override
	public Integer getFetchSize() 
	{
		return query.getFetchSize();
	}

	@Override
	public int getMaxResults() 
	{
		return query.getMaxResults();
	}

	@Override
	public int getFirstResult() 
	{
		return query.getFirstResult();
	}

	@Override
	public LockOptions getLockOptions() 
	{
		return query.getLockOptions();
	}

	@Override
	public String getComment() 
	{
		return query.getComment();
	}

	@Override
	public Query<R> addQueryHint(String hint) 
	{
		query.addQueryHint(hint);
		return this;
	}

	@Override
	public Map<String, Object> getHints() 
	{
		return query.getHints();
	}

	@Override
	public LockModeType getLockMode() 
	{
		return query.getLockMode();
	}

	@Override
	public <T> Parameter<T> getParameter(int position, Class<T> type)
	{
		return query.getParameter(position, type);
	}

	@Override
	public Parameter<?> getParameter(int position)
	{
		return query.getParameter(position);
	}

	@Override
	public <T> Parameter<T> getParameter(String name, Class<T> type) 
	{
		return query.getParameter(name, type);
	}

	@Override
	public Parameter<?> getParameter(String name) 
	{
		return query.getParameter(name);
	}

	@Override
	public Object getParameterValue(int position) 
	{
		return query.getParameterValue(position);
	}

	@Override
	public <T> T getParameterValue(Parameter<T> param) 
	{
		return query.getParameterValue(param);
	}

	@Override
	public Object getParameterValue(String name) 
	{
		return query.getParameterValue(name);
	}

	@Override
	public Set<Parameter<?>> getParameters() 
	{
		return query.getParameters();
	}

	@Override
	public boolean isBound(Parameter<?> param) 
	{
		return query.isBound(param);
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> T unwrap(Class<T> cls) 
	{
		return cls == this.getClass() ? (T)this : query.unwrap(cls);
	}

	@Override
	@Deprecated
	public Type determineProperBooleanType(int position, Object value, Type type) 
	{
		return query.determineProperBooleanType(position, value, type);
	}

	@Override
	@Deprecated
	public Type determineProperBooleanType(String name, Object value, Type type) 
	{
		return query.determineProperBooleanType(name, value, type);
	}

	@Override
	public FlushMode getHibernateFlushMode() 
	{
		return query.getHibernateFlushMode();
	}

	@Override
	public ParameterMetadata getParameterMetadata() 
	{
		return query.getParameterMetadata();
	}

	@Override
	public QueryProducer getProducer() 
	{
		return query.getProducer();
	}

	@Override
	public RowSelection getQueryOptions() 
	{
		return query.getQueryOptions();
	}

	@Override
	public Query<R> setFlushMode(FlushModeType flushMode) 
	{
		query.setFlushMode(flushMode);
		return this;
	}

	@Override
	public Query<R> setHibernateFlushMode(FlushMode flushMode) 
	{
		query.setHibernateFlushMode(flushMode);
		return this;
	}

	@Override
	public Query<R> setHint(String name, Object value) 
	{
		query.setHint(name, value);
		return this;
	}

	@Override
	public Query<R> setLockMode(LockModeType lockMode) 
	{
		query.setLockMode(lockMode);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, Calendar value, TemporalType type) 
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, Date value, TemporalType type) 
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, Instant value, TemporalType type)
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, LocalDateTime value, TemporalType type)
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, OffsetDateTime value, TemporalType type)
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public <P> Query<R> setParameter(int position, P value, TemporalType type)
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(int position, ZonedDateTime value, TemporalType type)
	{
		query.setParameter(position, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<Calendar> param, Calendar value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<Date> param, Date value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<Instant> param, Instant value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<LocalDateTime> param, LocalDateTime value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<OffsetDateTime> param, OffsetDateTime value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public <T> Query<R> setParameter(Parameter<T> param, T value) 
	{
		query.setParameter(param, value);
		return this;
	}

	@Override
	public Query<R> setParameter(Parameter<ZonedDateTime> param, ZonedDateTime value, TemporalType type)
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public <P> Query<R> setParameter(QueryParameter<P> param, P value, TemporalType type) 
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public <P> Query<R> setParameter(QueryParameter<P> param, P value, Type type) 
	{
		query.setParameter(param, value, type);
		return this;
	}

	@Override
	public <T> Query<R> setParameter(QueryParameter<T> param, T value) 
	{
		query.setParameter(param, value);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, Calendar value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, Date value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, Instant value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, LocalDateTime value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, OffsetDateTime value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public <P> Query<R> setParameter(String name, P value, TemporalType type) 
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public Query<R> setParameter(String name, ZonedDateTime value, TemporalType type)
	{
		query.setParameter(name, value, type);
		return this;
	}

	@Override
	public <P> Query<R> setParameterList(QueryParameter<P> param, Collection<P> values) 
	{
		query.setParameterList(param, values);
		return this;
	}

	@Override
	public Stream<R> stream() 
	{
		return query.stream();
	}

	@Override
	public Optional<R> uniqueResultOptional()
	{
		return query.uniqueResultOptional();
	}

}
