/*
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.EqualsHelper;
import net.sf.basedb.util.filter.Filter;

/**
	

	@author Nicklas
	@since 3.7
	@see Restrictions#conditionalRestriction(Filter, Restriction, Restriction, boolean)
*/
public class ConditionalQueryElement
	implements Restriction, Expression
{
	private final Filter<Query> filter;
	private final QueryElement ifTrue;
	private final QueryElement ifFalse;

	ConditionalQueryElement(Filter<Query> filter, QueryElement ifTrue, QueryElement ifFalse)
	{
		assert filter != null : "filter == null";
		assert ifTrue != null : "ifTrue == null";
		assert ifFalse != null : "ifFalse == null";
		this.filter = filter;
		this.ifTrue = ifTrue;
		this.ifFalse = ifFalse;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return filter.evaluate(query) ? ifTrue.toQl(query, dc) : ifFalse.toQl(query, dc);
	}
	/**
		@return The expression
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(ifTrue, ifFalse);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "(" + ifTrue.toString() + " || " + ifFalse.toString() + ")";
	}
	/**
		This query element is equal to another ConditionalQueryElement if their 
		filter and child elements are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		ConditionalQueryElement o = (ConditionalQueryElement)other;
		return EqualsHelper.equals(filter, o.filter) && EqualsHelper.equals(ifTrue, o.ifTrue) && EqualsHelper.equals(ifFalse, o.ifFalse);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(filter, ifTrue, ifFalse);
	}
	// -------------------------------------------
}

