/**
	$Id: QuadraticMeanExpression.java 5320 2010-04-21 09:48:29Z nicklas $

	Copyright (C)

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core.query;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.EqualsHelper;

import java.util.Collection;
import java.util.Collections;

/**
 	Calculates the quadratic mean of an expression 
  	@author Martin
  	@version 2.8
  	@see Aggregations#quadraticMean(Expression)
	@base.modified $Date: 2007-08-17 09:18:29 +0200 (Fri, 17 Aug 2007) $
 	
 */
public class QuadraticMeanExpression 
	implements Expression
{
	private final Expression e;
	
	private Expression sqrt;
	
	QuadraticMeanExpression(Expression e)
	{
		assert e != null : "e == null";
		this.e = e;
	}

	@Override
	/**
		@return The expression to find the quadratic mean for
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e);
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		return getSqrt().toQl(query, dc);
	}
	
	/*
	 	From the Object class
	 	-------------------------------
	*/
	@Override
	public String toString()
	{
		return getSqrt().toString();
	}
	/**
		This expression is equal to another QuadraticMeanExpression if their 
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		QuadraticMeanExpression o = (QuadraticMeanExpression)other;
		return EqualsHelper.equals(e, o.e);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(e) + getClass().hashCode();
	}
	// -------------------------------------------

	/*
	 	Gets the expression. Create a new if it not exists already.
	 */
	private Expression getSqrt()
	{
		if (sqrt == null)
		{
			sqrt = new SqrtExpression(new MeanExpression(new MultiplyExpression(e,e)));
		}
		return sqrt;
	}
}