/**
	$Id: MatrixBaseFileExporter.java 5689 2011-08-11 09:09:06Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import java.util.List;

import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.util.ChainedProgressReporter;
import net.sf.basedb.util.basefile.BaseFileWriter;

/**
	Exporter implementation that exports bioassay set data as a matrix
	BASEfile.

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2011-08-11 11:09:06 +0200 (to, 11 aug 2011) $
*/
public class MatrixBaseFileExporter
	extends BaseFileExporter
{

	public static final String SUBTYPE = "matrix";

	private BaseFileWriter out;
	private List<BioAssay> bioAssays;
	private DynamicSpotQuery spotQuery;
	private DynamicSpotQuery countQuery;
	private ChainedProgressReporter progress;
	private long spotCount;
	
	/**
		Create a new matrix BASEfile exporter.
	*/
	public MatrixBaseFileExporter()
	{}

	/**
		From AbstractBioAssaySetExporter/BaseFileExporter class
		-------------------------------------------------------
	*/
	/**
		Prepare the export by pre-loading some information and
		configure the queries that we are going to use.
	*/
	@Override
	protected void beginExport()
	{
		super.beginExport();
		bioAssays = getBioAssays();
		out = getBaseFileWriter();
		spotQuery = getSpotQuery(false);
		countQuery = getCountQuery(false);
		ProgressReporter p = getProgressReporter();
		if (p != null) progress = new ChainedProgressReporter(p);
		setProgress(0, "Caching reporter data...");
		cacheReporterData();
	}

	/**
		Exports headers for a single 'spots' section.
	*/
	@Override
	protected void exportSectionHeader()
	{
		// Get configuration options
		DbControl dc = getDbControl();
		
		// Count the spots in this section. Delegate output to superclass
		setProgress(10, "Counting number of spots...");
		spotCount = countQuery.count(dc);
		exportSpotSectionHeaders(bioAssays, spotCount);
	}

	/**
		Exports data for all bioassays in a single 'spots' section.
	*/
	@Override
	protected void exportSectionData()
	{
		if (progress != null) progress.setRange(20, 100);
		exportSpotSectionData(bioAssays, spotQuery, progress, spotCount);
	}

	/**
		Ends the 'spots' section.
		@return Always FALSE, since there are no more spot sections
	*/
	@Override
	protected boolean exportSectionFooter()
	{
		out.baseEndSection();
		out.flush();
		return false;
	}
	
	@Override
	protected void endExport(RuntimeException e)
	{
		if (e == null) setProgress(100, "Export complete. " + spotCount + " spots done");
		out = null;
		bioAssays = null;
		spotQuery = null;
		countQuery = null;
		progress = null;
		super.endExport(e);
	}
	// -------------------------------------------------------

	
}
