/**
	$Id: OneOfFilter.java 4594 2008-10-21 07:20:40Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.filter;

import java.util.Collection;

/**
	A filter implementation that accpets an object if 
	at least one parent filters accepts it.
	
	@author Nicklas
	@version 2.9
	@base.modified $Date: 2008-10-21 09:20:40 +0200 (ti, 21 okt 2008) $
*/
public class OneOfFilter<T>
	implements Filter<T>
{
	private final Collection<? extends Filter<? super T>> parents;

	/**
		Create a new filter.
		@param parents The parent filters
	*/
	public OneOfFilter( Collection<? extends Filter<? super T>> parents)
	{
		this.parents = parents;
	}

	/**
		@return TRUE if at least one parent filters returns TRUE, FALSE otherwise
	*/
	@Override
	public boolean evaluate(T object)
	{
		for (Filter<? super T> pf : parents)
		{
			if (pf.evaluate(object)) return true;
		}
		return false;
	}
}
