/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.Set;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer that transform extract items to their parent
	samples. This implementation will only transform to the extract:s immediate
	parent samples. Grandparents are not included (neither on the extract or
	sample side).
	
	To include all grandparents, combine this transformer with 
	{@link SampleToParentSampleTransformer} and {@link ExtractToParentExtractTransformer}.
	
	@author Nicklas
	@since 3.5
*/
public class ExtractToSampleTransformer 
	extends AbstractSourceItemTransformer 
{

	public ExtractToSampleTransformer()
	{
		super(Item.EXTRACT, Item.SAMPLE);
	}
	
	/**
		Load the ID of all samples that are immediate parents to the
		given set of extracts.
	*/
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		ItemQuery<Sample> query = Sample.getQuery();
		query.setIncludes(context.getInclude());
		query.join(Hql.innerJoin("children", "ch"));
		query.restrict(
			Restrictions.in(
				Hql.alias("ch"), 
				Expressions.parameter("extracts")
			)
		);
		return safeIdList(context.getDbControl(), query, "extracts", source);
	}


}
