/*
 $Id: TestExternalProgramExecutor.java 5384 2010-08-13 08:50:27Z nicklas $

 Copyright (C) 2010 Nicklas Nordborg

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.plugins.executor.ExternalProgramExecutor;
import net.sf.basedb.plugins.util.Parameters;


public class TestExternalProgramExecutor
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		write("++Testing External program executor");
		
		try
		{
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			int rootBasId = TestAnalyzeUtil.getItemId("bioassayset.root");
			int cloneId = TestAnalyzeUtil.clone_bioassay_set(rootBasId, "External executor clone");
			if (experimentId == 0 || rootBasId == 0 || cloneId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
				return ok;
			}
			
			int directoryId = TestDirectory.test_create(true, "External program executor");
			int pluginDefinitionId = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.ExternalProgramExecutor");
			int bfsExporter = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.BfsExporterSupport");
			int bfsImporter = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.BfsImporterSupport");
			int baseFileExporter = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.BaseFileExporterSupport");
			int baseFileImporter = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.BaseFileImporterSupport");
			int fileOnlyImporter = TestPluginDefinition.test_get("net.sf.basedb.plugins.executor.FileOnlyImporterSupport");

			String program = TestUtil.isWindows() ? "executeclass.bat" : "executeclass.sh";
			// If this parameter is TRUE temporary files are not deleted by the test program
			TestJob.Parameter debugMode = 
				new TestJob.Parameter(ExternalProgramExecutor.PARAMETER_DEBUG_MODE, false);

			// ********* BFS tests ************
			TestJob.Parameter reporterFields = new TestJob.Parameter(
				ExternalProgramExecutor.NAMESPACE_EXPORTER + ".reporterFields", "=property(\"Name\")");
			TestJob.Parameter assayFields = new TestJob.Parameter(
				ExternalProgramExecutor.NAMESPACE_EXPORTER + ".assayFields", "=property(\"Name\")");
			TestJob.Parameter spotFields = new TestJob.Parameter(
				ExternalProgramExecutor.NAMESPACE_EXPORTER + ".spotFields", "=jep(\"ch(1)\", \"Ch 1\", \"int\")\n=jep(\"ch(2)\", \"Ch 2\", \"int\")");
			TestJob.Parameter enableExtraFiles = new TestJob.Parameter(
				ExternalProgramExecutor.NAMESPACE_IMPORTER + ".extraFiles", true);

			int cloneBFS = test_create_configuration(pluginDefinitionId, "BFS copy", 
				program, "CopyStdin2Stdout", bfsExporter, bfsImporter, 
				debugMode, reporterFields, spotFields, assayFields);
			int errorBFS = test_create_configuration(pluginDefinitionId, "BFS error", 
				program, "CopyStdin2Stdout error", bfsExporter, bfsImporter, 
				debugMode, reporterFields, spotFields, assayFields);
			int extraFileBFS = test_create_configuration(pluginDefinitionId, "BFS extra", 
				program, "BfsExtraFileGenerator", bfsExporter, bfsImporter, 
				debugMode, reporterFields, spotFields, assayFields, enableExtraFiles);
			int extraFileNoSdataBFS = test_create_configuration(pluginDefinitionId, "BFS extra (no sdata)", 
				program, "BfsExtraFileGenerator nosdata", bfsExporter, bfsImporter, 
				debugMode, reporterFields, spotFields, assayFields, enableExtraFiles);
			
			// Clone bioassay set
			int copyBfsJob = test_create_job(cloneBFS, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Cloned from parent (BFS)"));
			// Simulate an error
			int errorBfsJob = test_create_job(errorBFS, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Should really not be created (BFS)"));
			// Add extra files to the result
			int extraFilesBfsJob = test_create_job(extraFileBFS, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Extra files+sdata (BFS)"),
				new TestJob.Parameter(ExternalProgramExecutor.NAMESPACE_IMPORTER + ".extraFilesDirectory", 
					"/External program executor/bfs-1"));
			// Result with only extra files
			int extraFilesNoSdataBfsJob = test_create_job(extraFileNoSdataBFS, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Extra files (no sdata) (BFS)"),
				new TestJob.Parameter(ExternalProgramExecutor.NAMESPACE_IMPORTER + ".extraFilesDirectory", 
				"/External program executor/bfs-2"));
			
			ok &= TestJob.test_execute(copyBfsJob, false);
			ok &= TestJob.test_execute(errorBfsJob, true);
			ok &= TestJob.test_execute(extraFilesBfsJob, false);
			ok &= TestJob.test_execute(extraFilesNoSdataBfsJob, false);
			
			// ********* BASEFile tests ************
			reporterFields = new TestJob.Parameter(
					ExternalProgramExecutor.NAMESPACE_EXPORTER + ".reporterFields", "Position\nInternal id\nName");
			int cloneBaseFile = test_create_configuration(pluginDefinitionId, "BASEfile copy", 
				program, "CopyStdin2Stdout", baseFileExporter, baseFileImporter, 
				debugMode, reporterFields, spotFields);
			int errorBaseFile = test_create_configuration(pluginDefinitionId, "BASEfile error", 
				program, "CopyStdin2Stdout error", baseFileExporter, baseFileImporter, 
				debugMode, reporterFields, spotFields);
			int extraBaseFile = test_create_configuration(pluginDefinitionId, "BASEfile extra-files", 
					program, "CopyStdin2Stdout extra", baseFileExporter, baseFileImporter, 
					debugMode, reporterFields, spotFields, enableExtraFiles);

			// Clone bioassay set
			int copyBaseFileJob = test_create_job(cloneBaseFile, experimentId, cloneId, debugMode,
					new TestJob.Parameter(Parameters.CHILD_NAME, "Cloned from parent (BASEfile)"));
			// Simulate an error
			int errorBaseFileJob = test_create_job(errorBaseFile, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Should really not be created (BASEfile)"));
			// Add extra files to the result
			int extraBaseFileJob = test_create_job(extraBaseFile, experimentId, cloneId, debugMode,
				new TestJob.Parameter(Parameters.CHILD_NAME, "Extra files (BASEFile)"),
				new TestJob.Parameter(ExternalProgramExecutor.NAMESPACE_IMPORTER + ".extraFilesDirectory", 
						"/External program executor/basefile-1"));

			ok &= TestJob.test_execute(copyBaseFileJob, false);
			ok &= TestJob.test_execute(errorBaseFileJob, true);
			ok &= TestJob.test_execute(extraBaseFileJob, false);

			// ********* File-only tests ************
			int cloneFileOnly = test_create_configuration(pluginDefinitionId, "File-only importer", 
					program, "BfsExtraFileGenerator", bfsExporter, fileOnlyImporter, 
					debugMode, reporterFields, spotFields, assayFields);
			int cloneFileOnlyJob = test_create_job(cloneFileOnly, experimentId, cloneId, debugMode,
					new TestJob.Parameter(Parameters.CHILD_NAME, "File-only importer"),
					new TestJob.Parameter(ExternalProgramExecutor.NAMESPACE_IMPORTER + ".extraFilesDirectory", 
					"/External program executor/fileonly-1"));
			ok &= TestJob.test_execute(cloneFileOnlyJob, false);
			
			if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
			
			// Cleanup
			TestAnalyzeUtil.test_delete_bioassayset(cloneId);
			TestJob.test_delete(copyBfsJob);
			TestJob.test_delete(errorBfsJob);
			TestJob.test_delete(extraFilesBfsJob);
			TestJob.test_delete(extraFilesNoSdataBfsJob);
			TestJob.test_delete(copyBaseFileJob);
			TestJob.test_delete(errorBaseFileJob);
			TestJob.test_delete(extraBaseFileJob);
			TestJob.test_delete(cloneFileOnlyJob);

			TestPluginConfiguration.test_delete(cloneBFS);
			TestPluginConfiguration.test_delete(errorBFS);
			TestPluginConfiguration.test_delete(extraFileBFS);
			TestPluginConfiguration.test_delete(extraFileNoSdataBFS);
			TestPluginConfiguration.test_delete(cloneBaseFile);
			TestPluginConfiguration.test_delete(errorBaseFile);
			TestPluginConfiguration.test_delete(extraBaseFile);
			TestPluginConfiguration.test_delete(cloneFileOnly);
			
			TestDirectory.test_delete(directoryId, true);
		}
		finally
		{
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
			write("++Testing External program executor "+(ok ? "OK" : "Failed")+"\n");
		}
		return ok;
	}
	

	static void write(String message)
	{
		System.out.println(message);
	}
	
	
	static int test_create_configuration(int pluginDefinitionId, String name, 
		String program, String cmdLine,
		int exporterPlugin, int importerPlugin, TestJob.Parameter... parameters)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition plugin = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginDefinition exporter = PluginDefinition.getById(dc, exporterPlugin);
			PluginDefinition importer = PluginDefinition.getById(dc, importerPlugin);
			
			PluginConfiguration config = plugin.newPluginConfiguration();
			config.setName(name);

			PluginConfigurationRequest request = config.configure();
			TestJob.write_request_information(request.getRequestInformation());
			TestJob.setRequestParameterValues(request, parameters);
			request.setParameterValue(ExternalProgramExecutor.PARAMETER_EXTERNAL_PATH, 
				new java.io.File("").getAbsolutePath()+java.io.File.separator + program);
			request.setParameterValue(ExternalProgramExecutor.PARAMETER_EXTERNAL_CMDLINE,
				cmdLine);
			request.setParameterValue(ExternalProgramExecutor.PARAMETER_EXPORTER_CLASS, 
				exporter.getClassName());
			request.setParameterValue(ExternalProgramExecutor.PARAMETER_IMPORTER_CLASS, 
				importer.getClassName());

			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.ERROR)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			else if (response.getStatus() != Response.Status.CONTINUE)
			{
				throw new BaseException(response.getStatus() + ": " + response.getMessage());
			}
			
			// Second step: configure exporter/importer
			request = response.getNextRequest();
			TestJob.write_request_information(request.getRequestInformation());
			TestJob.setRequestParameterValues(request, parameters);
			response = request.invoke();
			
			if (response.getStatus() == Response.Status.ERROR)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			else if (response.getStatus() != Response.Status.CONTINUE)
			{
				throw new BaseException(response.getStatus() + ": " + response.getMessage());
			}
		
			// Third step: program parameters
			request = response.getNextRequest();
			TestJob.write_request_information(request.getRequestInformation());
			TestJob.setRequestParameterValues(request);
			response = request.invoke();
			
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(config);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = config.getId();
			write("--Create configuration for External program executor OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for External program executor FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_job(int pluginConfigurationId, int experimentId, int bioAssaySetId, 
			TestJob.Parameter... parameters)
	{
		if (pluginConfigurationId == 0 || experimentId == 0 || bioAssaySetId == 0 
				|| !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			Experiment e = Experiment.getById(dc, experimentId);
			BioAssaySet bioAssaySet = BioAssaySet.getById(dc, bioAssaySetId);
			dc.getSessionControl().getCurrentContext(Item.EXPERIMENT).setId(experimentId);
			dc.getSessionControl().getCurrentContext(Item.BIOASSAYSET).setId(bioAssaySetId);
			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(e);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			
			// First, set default values
			TestJob.setRequestParameterValues(request);
			
			// Then, set values from method call
			if (parameters != null)
			{
				for (TestJob.Parameter p : parameters)
				{
					request.setParameterValue(p.name, p.value);
				}
			}
			
			// At last, some well-known parameters
			request.setParameterValue("source", bioAssaySet);
			TestJob.write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for External program executor OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for External program executor FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
}
