/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Extracts = function()
{
	var extracts = {};
	
	/**
		Initialize the page.
	*/
	extracts.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', extracts.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', extracts.loadAnnotationsFrame);
			TabControl.addTabValidator('settings.info', extracts.validateExtract);

			// Quantity
			Events.addEventHandler('original_quantity', 'keypress', Events.numberOnly);

			// Tag
			Buttons.addClickHandler('tag_id.select', extracts.selectTag);
			Events.addEventHandler('tag_id', 'base-selected', Items.onItemSelected);

			// Protocol
			Buttons.addClickHandler('protocol_id.select', extracts.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', Items.onItemSelected);
			// Kit
			Buttons.addClickHandler('kit_id.select', extracts.selectKit);
			Events.addEventHandler('kit_id', 'base-selected', Items.onItemSelected);
			// Subtype
			Events.addEventHandler('subtype_id', 'change', extracts.subtypeOnChange);

			// Bioplate and biowell
			Buttons.addClickHandler('bioplate_id.select', extracts.selectBioPlate);
			Events.addEventHandler('bioplate_id', 'base-selected', Items.onItemSelected);
			Events.addEventHandler('bioplate_id', 'change', extracts.bioPlateOnChange);
			Buttons.addClickHandler('biowell_id.select', extracts.selectBioWell);
			Events.addEventHandler('biowell_id', 'base-selected', Items.onItemSelected);
			extracts.initBioWell();

			// Parent items
			Events.addEventHandler('parentType.sample', 'click', extracts.parentTypeOnClick);
			Events.addEventHandler('parentType.extract', 'click', extracts.parentTypeOnClick);
			Buttons.addClickHandler('sample_id.select', extracts.selectSample);
			Events.addEventHandler('sample_id', 'base-selected', Items.onItemSelected);
			Events.addEventHandler('sample_id', 'base-selected', extracts.sampleOnChange);
			Events.addEventHandler('used_from_sample', 'keypress', Events.numberOnly);
			Events.addEventHandler('used_from_sample', 'blur', extracts.usedFromSampleOnBlur);

			Events.addEventHandler('extracts', 'change', extracts.extractsOnChange);
			Buttons.addClickHandler('btnAddExtracts', extracts.addExtractsOnClick);
			Events.addEventHandler('btnAddExtracts', 'base-selected', extracts.addExtractCallback);
			
			Events.addEventHandler('used_quantity', 'keypress', Events.numberOnly);
			Events.addEventHandler('used_quantity', 'keyup', extracts.usedQuantityOnChange);

			extracts.parentTypeOnClick();

		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'EXTRACT', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnNewExtract', extracts.newExtract, attributes);
			Buttons.addClickHandler('btnNewPhysicalBioAssay', extracts.newPhysicalBioAssay, attributes);
			Buttons.addClickHandler('btnNewRawBioAssay', extracts.newRawBioAssay, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.events', extracts.viewEvents);
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'EXTRACT'};
			var tableAttributes = {'table-id': 'extracts'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnInheritAnnotations', Buttons.inheritAnnotations, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewItemList', extracts.newItemListOnClick);
			Buttons.addClickHandler('btnPlaceOnPlate', extracts.placeOnPlateOnClick);
			Buttons.addClickHandler('btnNewPooledItem', extracts.newPooledItemOnClick);
			Buttons.addClickHandler('btnNewPhysicalBioAssay', extracts.newPhysicalBioAssayFromTable);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	// Add event handlers to the 'new sample' icons
	extracts.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-extract')
		{
			Buttons.addClickHandler(element, extracts.newExtract);
		}
		else if (autoInit == 'new-raw-bioassay')
		{
			Buttons.addClickHandler(element, extracts.newRawBioAssay);
		}
	}

	/*
		Create a new extract with the current extract as the parent
	*/
	extracts.newExtract = function(event)
	{
		var extractId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('EXTRACT', '&extract_id='+extractId);
	}
	
	/*
		Create a new raw bioassay with the current extract as the parent
	*/
	extracts.newRawBioAssay = function(event)
	{
		var extractId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('RAWBIOASSAY', '&extract_id='+extractId);
	}
	
	/*
		Create a new physical bioassay with the current extract as the parent
	*/
	extracts.newPhysicalBioAssay = function(event)
	{
		var extractId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('PHYSICALBIOASSAY', '&extract_id='+extractId);
	}

	// Create a new item list with the selected items as members
	extracts.newItemListOnClick = function()
	{
		var controller = Items.getController('ITEMLIST');
		Table.submitToPopup('extracts', 'CreateItemList', controller.width, controller.height);
	}
	
	extracts.placeOnPlateOnClick = function()
	{
		Table.submitToPopup('extracts', 'PlaceOnPlate', 900, 600);
	}
	
	extracts.newPooledItemOnClick = function()
	{
		if (!Table.checkIfSelected('extracts'))
		{
			return;
		}
		Table.submitToPopup('extracts', 'NewPooledItem', 750, 500);
	}

	extracts.newPhysicalBioAssayFromTable = function()
	{
		if (!Table.checkIfSelected('extracts'))
		{
			return;
		}
		Table.submitToPopup('extracts', 'NewPhysicalBioAssay', 750, 500);
	}
	
	extracts.viewEvents = function()
	{
		var extractId = Data.get('page-data', 'item-id');
		var url = '../events/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&biomaterial_id='+extractId;
		url += '&biomaterial_type=EXTRACT';
		location.replace(url);
	}
	
	extracts.validateExtract = function()
	{
		var frm = document.forms['extract'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (Strings.trim(frm.bioplate_id.value) != 0 && Strings.trim(frm.biowell_id.value) == 0)
		{
			Forms.showNotification('biowell_id.select', 'You must choose a biowell from the bioplate');
			return false;
		}
		return true;
	}

	extracts.save = function()
	{
		var frm = document.forms['extract'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Link.exportActions('extracts');
			frm.submit();
		}
	}


	extracts.loadAnnotationsFrame = function()
	{
		var frm = document.forms['extract'];
		var protocolId = 0;
		if (frm.protocol_id.length > 0 && !frm.protocol_id.disabled) 
		{
			protocolId = Math.abs(parseInt(frm.protocol_id.value));				
		}
		Annotations.autoLoadEditFrame(protocolId, ItemSubtype.getSubtypeId('subtype_id'), extracts.getParents());
	}
	
	extracts.getParents = function()
	{
		var frm = document.forms['extract'];
		var parents = [];
		if (frm.parentType[0].checked)
		{
			var sampleId = Math.abs(parseInt(frm.sample_id.value));
			if (sampleId > 0) parents[parents.length] = 'SAMPLE:'+sampleId;
		}
		else
		{
			var ids = Link.getIdsInList(frm.extracts, 'EXTRACT');
			if (ids.length > 0)
			{
				parents[parents.length] = 'EXTRACT:'+ids.join(':');
			}
		}
		var tagId = Math.abs(parseInt(frm.tag_id.value));
		if (tagId > 0) parents[parents.length] = 'TAG:'+tagId;
		var kitId = Math.abs(parseInt(frm.kit_id.value));
		if (kitId > 0) parents[parents.length] = 'KIT:'+kitId;
		var protocolId = Math.abs(parseInt(frm.protocol_id.value));
		if (protocolId > 0) parents[parents.length] = 'PROTOCOL:'+protocolId;
		return parents;
	}

	extracts.subtypeOnChange = function()
	{
		var frm = document.forms['extract'];
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		var recentInfo = ItemSubtype.getRelatedProjectDefaultAndRecentItems('EXTRACT', subtypeId, ['PROTOCOL', 'KIT', 'BIOPLATE', 'TAG', 'SAMPLE', 'EXTRACT']);
		ItemSubtype.updateSelectionList(frm.protocol_id, recentInfo.PROTOCOL['recent'], recentInfo.PROTOCOL['default']);
		ItemSubtype.updateSelectionList(frm.kit_id, recentInfo.KIT['recent']);
		ItemSubtype.updateSelectionList(frm.tag_id, recentInfo.TAG['recent']);
		ItemSubtype.updateSelectionList(frm.bioplate_id, recentInfo.BIOPLATE['recent']);
	}
	
	extracts.selectTag = function()
	{
		var frm = document.forms['extract'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'TAG');
		if (frm.tag_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.tag_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('TAG', 'tag_id', 0, url);
	}
	
	extracts.selectProtocol = function()
	{
		var frm = document.forms['extract'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'PROTOCOL', Data.int('page-data', 'extraction-protocoltype-id'));
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	extracts.selectKit = function()
	{
		var frm = document.forms['extract'];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:BOOLEAN:inactive=0';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'KIT', Data.int('page-data', 'extraction-kittype-id'));
		if (frm.kit_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.kit_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('KIT', 'kit_id', 0, url);
	}
	
	extracts.selectBioPlate = function()
	{
		var frm = document.forms['extract'];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:bioPlateType.bioMaterialType='+encodeURIComponent('|'+Data.get('page-data', 'extract-type'));
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		// Restrict to plates with the given subtype
		url += '&tmpfilter:INT:bioPlateType.itemSubtype='+encodeURIComponent(subtypeId ? '|' + subtypeId : '=');
		url += '&tmpfilter:BOOLEAN:destroyed=false';
		Dialogs.selectItem('BIOPLATE', 'bioplate_id', 0, url);
	}
	
	extracts.bioPlateOnChange = function()
	{
		var frm = document.forms['extract'];
		var list = frm.bioplate_id;
		frm.biowell_id.selectedIndex=0;
		frm.biowell_id.remove(1);			
	}
	
	extracts.initBioWell = function()
	{
		var biowell = JSON.parse(Data.get('page-data', 'biowell'));
		if (biowell)
		{
			var frm = document.forms['extract'];
			var option = new Option(biowell.location, -biowell.id, true, true);
			frm.biowell_id[1] = option;
		}
	}
	
	extracts.selectBioWell = function()
	{
		var frm = document.forms['extract'];
		var bioplateId = Math.abs(parseInt(frm.bioplate_id.value))
		if (bioplateId == 0)
		{
			Forms.showNotification('bioplate_id.select', 'You must first select a bioplate', null, 'pointer-below');
			return;
		}
		
		var url = '&bioplate_id='+bioplateId;
		url += '&resetTemporary=1&tmpfilter:STRING:$mbm.name='+escape('=');
		url += '&tmpfilter:STRING:originalBioMaterial.name='+escape('=');
		url += '&columns=row,column';

		Dialogs.selectItem('BIOWELL', 'biowell_id', 0, url);
	}
	
	extracts.parentTypeOnClick = function()
	{
		var frm = document.forms['extract'];
		var useExtracts = frm.parentType[1].checked;
		frm.sample_id.disabled = useExtracts;
		frm.used_from_sample.disabled = useExtracts;
		frm.extracts.disabled = !useExtracts;
		frm.used_quantity.disabled = !useExtracts;
	}

	extracts.selectSample = function()
	{
		var frm = document.forms['extract'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'SAMPLE');
		if (frm.sample_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.sample_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('SAMPLE', 'sample_id', 0, url);
	}
	
	extracts.sampleOnChange = function(event)
	{
		var frm = document.forms['extract'];
		frm.parentType[0].checked = true;
		frm.parentType[1].checked = false;
		extracts.parentTypeOnClick();
	}

	extracts.usedFromSampleOnBlur = function()
	{
		var frm = document.forms['extract'];
		var sampleId = parseInt(frm.sample_id.value);
		if (sampleId < 0)
		{
			frm.sample_id[frm.sample_id.selectedIndex].value = -sampleId;
		}
	}
	
	extracts.addExtractsOnClick = function(event)
	{
		var frm = document.forms['extract'];
		var currentExtracts = Link.getIdsInList(frm.extracts, 'EXTRACT');
		var url = '&resetTemporary=1&exclude='+currentExtracts.join(',');
		url += ItemSubtype.createRelatedFilter('subtype_id', 'EXTRACT');
		Dialogs.selectItem('EXTRACT', event.currentTarget.id, 1, url);
	}

	extracts.addExtractCallback = function(event)
	{
		var frm = document.forms['extract'];
		event.detail.name += ' [-]';
		Link.addItem('extracts', 'EXTRACT', event.detail);
		if (event.detail.remaining == 0)
		{
			frm.parentType[0].checked = false;
			frm.parentType[1].checked = true;
			extracts.parentTypeOnClick();
		}
	}

	extracts.usedQuantityOnChange = function()
	{
		var frm = document.forms['extract'];
		var usedQuantity = frm.used_quantity.value;
		var displayQuantity = usedQuantity == '' ? '-' : usedQuantity+' µg';
		for (var i = 0; i < frm.extracts.length; i++)
		{
			var option = frm.extracts[i];
			if (option.selected && option.item.id)
			{
				option.item.value = usedQuantity;
				var text = option.text.replace(/\[.*\]/, '['+displayQuantity+']');
				option.text = text; 
			}
		}
	}
	
	extracts.extractsOnChange = function()
	{
		var frm = document.forms['extract'];
		var item = frm.extracts[frm.extracts.selectedIndex].item;
		if (item && item.id && item.value != undefined)
		{
			frm.used_quantity.value = item.value;
		}
		else
		{
			frm.used_quantity.value = '';
		}
		frm.used_quantity.focus();
	}

	return extracts;
}();

Doc.addElementInitializer(Extracts.initElements);
Doc.onLoad(Extracts.initPage);
