/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var EditFile = function()
{
	var editFile = {};
	var originalText = '';
	var isModified = false;
	
	editFile.initPage = function()
	{
		editFile.loadFile();
		
		// Save + Close buttons
		Buttons.addClickHandler('btnSave', editFile.save);
		Buttons.addClickHandler('close', editFile.close);

		Events.addEventHandler('filedata', 'scroll', editFile.updateLineNumbers);
		Events.addEventHandler('filedata', 'keydown', editFile.detectTab);
		Events.addEventHandler('filedata', 'keyup', editFile.detectChanges);
	}
	
	editFile.loadFile = function()
	{
		var path = Data.get('page-data', 'path');
		var url = 'download/-'+App.getSessionId()+'-'+path+'?'+(new Date().getTime());
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, editFile.onFileLoaded);
		request.send(null);
	}

	editFile.onFileLoaded = function(request)
	{
		var frm = document.forms['file'];
		frm.filedata.value = request.responseText;
		originalText = frm.filedata.value;
		editFile.setModified(false);
		editFile.updateLineNumbers();
	}

	editFile.setModified = function(modified)
	{
		isModified = modified;
		Doc.element('modified').innerHTML = isModified ? '*' : '';
	}
	
	editFile.updateLineNumbers = function()
	{
		var frm = document.forms['file'];
		var totalScroll = frm.filedata.scrollHeight;
		var topScroll = frm.filedata.scrollTop;
		var textHeight = frm.filedata.offsetHeight;
		var topLine = 1;
		var scrollRemain = 0;
		if (totalScroll > textHeight || topScroll > 0)
		{
			var numLines = frm.filedata.value.split('\n').length;
			var scrollPerLine = totalScroll / numLines;
			topLine = 1 + Math.floor(topScroll / scrollPerLine);
			scrollRemain = topScroll - scrollPerLine * (topLine - 1);
		}
		var lineNumberHTML = '';
		for (var i = topLine; i < topLine + 100; i++)
		{
			lineNumberHTML += i + '<br>';
		}
		var lineNumberElem = Doc.element('linenumbers');
		lineNumberElem.innerHTML = lineNumberHTML;
		lineNumberElem.scrollTop = scrollRemain;
	}

	editFile.save = function()
	{
		if (isModified)
		{
			editFile.saveFile();
		}
		else
		{
			editFile.closeWindow(false);
		}
	}
	
	editFile.close = function()
	{
		if (isModified)
		{
			if (confirm('The file has been modified. Save changes?'))
			{
				editFile.saveFile();
			}
		}
		editFile.closeWindow(false);
	}
	
	editFile.saveFile = function()
	{
		var frm = document.forms['file'];
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=StoreFile&item_id='+Data.get('page-data', 'file-id');
		url += '&size='+frm.filedata.value.length;
		var request = Ajax.getXmlHttpRequest();
		request.open("POST", url, true);
		Ajax.setReadyStateHandler(request, editFile.onFileStored);
		request.send(frm.filedata.value);
	}

	editFile.onFileStored = function(request)
	{
		var response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			Forms.showNotification('btnSave', response.message);
			return;
		}
		editFile.setModified(false);
		editFile.closeWindow(true, response.message);
	}

	editFile.closeWindow = function(refreshParent, message)
	{
		if (refreshParent)
		{
			var url = App.getRoot() + 'common/close_popup.jsp?refresh_opener=1';
			if (message) url += '&message='+encodeURIComponent(message);
			location.href = url;
		}
		else
		{
			window.close();
		}
	}

	editFile.detectChanges = function()
	{
		if (!isModified)
		{
			var frm = document.forms['file'];
			if (frm.filedata.value != originalText)
			{
				editFile.setModified(true);
			}
		}
	}

	
	editFile.detectTab = function(event)
	{
		if (event.keyCode == 9)
		{
			var frm = document.forms['file'];
			editFile.replaceSelection(frm.filedata, '\t');
			event.preventDefault();
			return false;
		}
		return true;
	}
	
	/*
		Replace the selected text with the given text. Code based on
		Trac: http://projects.edgewall.com/trac/
	*/
	editFile.replaceSelection = function(field, text)
	{
		field.focus();
	    var start, end, sel, scrollPos;
	    if (typeof(document["selection"]) != "undefined") 
	    {
	      sel = document.selection.createRange().text;
	    } 
	    else if (typeof(field["setSelectionRange"]) != "undefined") 
	    {
	      start = field.selectionStart;
	      end = field.selectionEnd;
	      scrollPos = field.scrollTop;
	      sel = field.value.substring(start, end);
	    }
	    if (sel.match(/ $/)) 
	    { 
	    	// exclude ending space char, if any
	      	sel = sel.substring(0, sel.length - 1);
	    }
	    if (typeof(document["selection"]) != "undefined") 
	    {
	      var range = document.selection.createRange().text = text;
	    } 
	    else if (typeof(field["setSelectionRange"]) != "undefined") 
	    {
	    	field.value = field.value.substring(0, start) + text +
	      		field.value.substring(end);
	      field.setSelectionRange(start + text.length, start + text.length);
	      field.scrollTop = scrollPos;
	    }
	}


	
	return editFile;
}();

Doc.onLoad(EditFile.initPage);
