<%-- $Id: analysis_tree.jsp 7337 2017-04-12 09:20:29Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Experiment"
	import="net.sf.basedb.core.BioAssaySet"
	import="net.sf.basedb.core.Transformation"
	import="net.sf.basedb.core.ExtraValue"
	import="net.sf.basedb.core.ExtraValueType"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.Job"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.PluginConfiguration"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.snapshot.AnnotationLoaderUtil"
	import="net.sf.basedb.core.snapshot.AnnotationTypeFilter"
	import="net.sf.basedb.core.snapshot.AnnotationSnapshot"
	import="net.sf.basedb.core.snapshot.AnnotationSetSnapshot"
	import="net.sf.basedb.core.snapshot.SnapshotManager"
	import="net.sf.basedb.util.Tree"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction"
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.util.ProjectSpecificInfoFilter"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.util.extensions.Renderer"
	import="java.util.Date"
	import="java.util.List"
	import="java.util.ArrayList"
	import="java.util.LinkedList"
	import="java.util.Map"
	import="java.util.HashMap"
	import="java.util.Iterator"
	import="java.util.Collection"
	import="org.json.simple.JSONObject"
	import="org.json.simple.JSONArray"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.BIOASSAYSET;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);
%>
<%!

private static void addToTree(Tree<BasicItem> tree, BioAssaySet bas)
{
	if (!tree.contains(bas))
	{
		Transformation transformation = bas.getTransformation();
		Tree.Entry<BasicItem> parent = tree.getEntry(transformation);
		if (parent == null) 
		{
			addToTree(tree, transformation);
			parent = tree.getEntry(transformation);
		}
		Tree.Entry<BasicItem> basEntry = parent.addChild(bas);
		for (ExtraValue xv : bas.getExtraValues().list(bas.getDbControl())) 
		{
			basEntry.addChild(xv);
		}
	}
}

private static void addToTree(Tree<BasicItem> tree, Transformation transformation)
{
	if (!tree.contains(transformation))
	{
		BioAssaySet source = transformation.getSource();
		Tree.Entry<BasicItem> parent = tree.getEntry(source);
		if (parent == null) 
		{
			addToTree(tree, source);
			parent = tree.getEntry(source);
		}
		parent.addChild(transformation);
	}
}

private static Tree<BasicItem> getAnalysisTree(DbControl dc, ItemQuery<BioAssaySet> bioAssaySetQuery, ItemQuery<Transformation> transformationQuery)
{
	Tree<BasicItem> tree = new Tree<BasicItem>(null);
	ItemResultList<BioAssaySet> allBioAssaySets = bioAssaySetQuery.list(dc);
	List<Integer> ids = new ArrayList<Integer>(allBioAssaySets.size());
	
	for (BioAssaySet bas : allBioAssaySets)
	{
		addToTree(tree, bas);
		ids.add(bas.getId());
	}
	
	transformationQuery.restrict(
		Restrictions.in(
			Hql.property("source.id"), 
			Expressions.parameter("bioAssaySets", ids)
		)
	);
	ItemResultList<Transformation> allTransformations = transformationQuery.list(dc);
	for (Transformation t: allTransformations)
	{
		addToTree(tree, t);
	}
	return tree;
}

JSONArray generateSubTree(JSONObject jsonParent, BasicItem parent, Tree<BasicItem> tree, Collection<String> closed)
{
	JSONArray json = new JSONArray();
	
	Tree.Entry<BasicItem> parentEntry = tree.getEntry(parent);
	if (parentEntry == null) return json;
	
	if (jsonParent == null && parent != null)
	{
		Item parentType = parent.getType();
		if (parentType == Item.BIOASSAYSET)
		{
			jsonParent = newJoustBioAssaySet(null, (BioAssaySet)parent, true);
		}
		else if (parentType == Item.TRANSFORMATION)
		{
			jsonParent = newJoustTransformation(null, (Transformation)parent, true);
		}
		if (jsonParent != null) 
		{
			jsonParent.put("noOutlineIcon", 1);
			json.add(jsonParent);
		}
	}
	
	List<Tree.Entry<BasicItem>> children = parentEntry.getChildren();
	if (children != null)
	{
		for (Tree.Entry<BasicItem> childEntry : children)
		{
			BasicItem child = childEntry.getNode();
			Item childType = child.getType();
			JSONObject jsonChild = null;
			String var = childType.name() + "_" + child.getId();
			boolean isOpen = (closed == null || !closed.contains(var)) && childEntry.getNumChildren() > 0;
			if (childType == Item.BIOASSAYSET)
			{
				jsonChild = newJoustBioAssaySet(jsonParent, (BioAssaySet)child, isOpen);
			}
			else if (childType == Item.TRANSFORMATION)
			{
				jsonChild = newJoustTransformation(jsonParent, (Transformation)child, isOpen);
			}
			else if (childType == Item.EXTRAVALUE)
			{
				jsonChild = newJoustExtraValue(jsonParent, (ExtraValue)child, isOpen);
			}
			if (jsonParent == null) json.add(jsonChild);
			generateSubTree(jsonChild, child, tree, closed);
		}
	}
	return json;
}


JSONObject newJoustBioAssaySet(JSONObject jsonParent, BioAssaySet bas, boolean open)
{
	String id = "BIOASSAYSET_"+bas.getId();
	String name = bas.getName();
	String icon = "BioAssaySet";
	
	JSONObject json = newJoustEntry(jsonParent, icon, HTML.encodeTags(name), id);
	json.put("type", "bioassayset");
	json.put("isOpen", open ? 1 : 0);
	return json;
}

JSONObject newJoustTransformation(JSONObject jsonParent, Transformation t, boolean open)
{
	String id = "TRANSFORMATION_"+t.getId();
	String name = t.getName();
	String icon = "Transformation";
	try
	{
		if (t.getJob().getPluginDefinition().supports("net.sf.basedb.core.plugin.AnalysisFilterPlugin"))
		{
			icon = "Filter";
		}
	}
	catch (Throwable tt)
	{}
	
	JSONObject json = newJoustEntry(jsonParent, icon, HTML.encodeTags(name), id);
	json.put("type", "transformation");
	json.put("isOpen", open ? 1 : 0);
	return json;
}

JSONObject newJoustExtraValue(JSONObject jsonParent, ExtraValue ev, boolean open)
{
	String id = "EXTRAVALUE_"+ev.getId();
	String name = ev.getValueType().toString();
	String icon = "ExtraValue";
	
	try
	{
		name = ev.getExtraValueType().getName();
	}
	catch (PermissionDeniedException ex)
	{}
	
	JSONObject json = newJoustEntry(jsonParent, icon, HTML.encodeTags(name), id);
	json.put("type", "extra-value");
	json.put("isOpen", open ? 1 : 0);
	return json;
}

JSONObject newJoustEntry(JSONObject jsonParent, String icon, String text, String id)
{
	JSONObject jsonJoust = new JSONObject();
	jsonJoust.put("icon", icon);
	jsonJoust.put("text", text);
	jsonJoust.put("id", id);
	if (jsonParent != null)
	{
		JSONArray jsonChildren = (JSONArray)jsonParent.get("children");
		if (jsonChildren == null)
		{
			jsonChildren = new JSONArray();
			jsonParent.put("children", jsonChildren);
			jsonParent.remove("isLazy");
		}
		jsonChildren.add(jsonJoust);
	}
	return jsonJoust;
}
%>
<%
final int experimentId = Values.getInt(request.getParameter("experiment_id"));
final int bioAssaySetId = Values.getInt(request.getParameter("item_id"));
final int transformationId = Values.getInt(request.getParameter("transformation_id"));
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final String rootPath = request.getContextPath()+"/";
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final ItemContext tc = sc.getCurrentContext(Item.TRANSFORMATION);
final ItemContext xvc = sc.getCurrentContext(Item.EXTRAVALUE);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("bioassay set", "bioassay sets");
final DbControl dc = sc.newDbControl();
Tree<BasicItem> analysisTree = null;
List<AnnotationLoaderUtil> annotationLoaders = new ArrayList<AnnotationLoaderUtil>();
try
{
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	final ItemQuery<AnnotationType> annotationTypeQuery = Base.getAnnotationTypesQuery(itemType);
	SnapshotManager manager = new SnapshotManager();
	ProjectSpecificInfoFilter psInfo = new ProjectSpecificInfoFilter();
	for (AnnotationType at : annotationTypeQuery.list(dc))
	{
		annotationLoaders.add(new AnnotationLoaderUtil(dc, manager, at));
	}
	final Experiment experiment = Experiment.getById(dc, experimentId);
	final BasicItem root = transformationId == 0 ? 
			(bioAssaySetId == 0 ? null : BioAssaySet.getById(dc, bioAssaySetId))
			: Transformation.getById(dc, transformationId);
	final boolean createPermission = experiment.hasPermission(Permission.USE);
	final boolean deletePermission = createPermission;
	final boolean writePermission = createPermission;

	final ItemQuery<Transformation> transformationQuery = experiment.getTransformations();
	transformationQuery.include(cc.getInclude());
	transformationQuery.setFirstResult(0);
	transformationQuery.setMaxResults(-1);
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	try
	{
		final ItemQuery<BioAssaySet> query = Base.getConfiguredQuery(dc, cc, true, experiment.getBioAssaySets(), mode);
		query.setFirstResult(0);
		query.setMaxResults(-1);
		analysisTree = getAnalysisTree(dc, query, transformationQuery);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
		t.printStackTrace();
	}
	// Contains the ID:s of the bioassaysets that are closed in the tree
	Collection<String> closed = (Collection<String>)cc.getObject("closed");
	int numListed = 0;
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, 
		guiContext, root == null ? experiment : root);
	ExtensionsInvoker toolsInvoker = ExtensionsControl.useExtensions(jspContext, 
			"net.sf.basedb.clients.web.bioassayset.list.tools");
	ExtensionsInvoker toolbarInvoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	
	JSONArray json = generateSubTree(null, root, analysisTree, closed);
	%>
	<base:page type="include">
	<base:head scripts="joust-2.js,/views/experiments/bioassaysets/analysis_tree.js" />
	<base:body>
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
		
		<div id="tree-data" class="datacontainer"
			data-experiment-id="<%=experimentId%>"
			data-bioassayset-id="<%=bioAssaySetId%>"
			data-transformation-id="<%=transformationId %>"
			data-joust-tree="<%=HTML.encodeTags(json.toJSONString()) %>"
		></div>
		
		<tbl:table 
			id="bioAssaySets" 
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="<%=transformationId != 0 ? "../bioassaysets/index.jsp" : "index.jsp"%>"
			sc="<%=sc%>"
			item="<%=itemType%>"
			filterrows="<%=cc.getFilterRows()%>"
			subclass="<%=root == null ? "fulltable" : "" %>"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="experiment_id" 
				value="<%=String.valueOf(experimentId)%>" 
			/>
			<tbl:hidden 
				name="item_id" 
				value="<%=String.valueOf(bioAssaySetId)%>"
				skip="<%=bioAssaySetId == 0%>"
			/>
			<tbl:hidden 
				name="transformation_id" 
				value="<%=String.valueOf(transformationId)%>"
				skip="<%=transformationId == 0%>"
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:hidden
				name="closed"
				value="<%=closed == null ? "," : ","+Values.getString(closed, ",", true)+","%>"
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="spots"
				property="numSpots"
				datatype="int"
				title="Spots/Values in db"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="reporters"
				property="numReporters"
				datatype="int"
				title="Reporters in db"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="fileSpots"
				property="numFileSpots"
				datatype="int"
				title="Spots/Values in file"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="fileReporters"
				property="numFileReporters"
				datatype="int"
				title="Reporters in file"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="date"
				property="transformation.job.ended"
				datatype="timestamp"
				title="Date"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="plugin"
				property="transformation.job.pluginDefinition.name"
				datatype="string"
				title="Plugin"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>
			<%
			for (AnnotationLoaderUtil loader : annotationLoaders)
			{
				AnnotationType at = loader.getAnnotationType();
				Enumeration<String, String> annotationEnum = null;
				Formatter formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
				if (at.isEnumeration())
				{
					annotationEnum = new Enumeration<String, String>();
					if (!at.getDisplayAsList()) annotationEnum.add("", "-none-");
					List<?> values = at.getValues();
					for (Object value : values)
					{
						String encoded = formatter.format(value);
						annotationEnum.add(encoded, encoded);
					}
				}
				%>
				<tbl:columndef 
					id="<%="at"+at.getId()%>"
					title="<%=HTML.encodeTags(at.getName())+" [A]"%>" 
					property="<%="#"+at.getId()%>"
					annotation="true"
					datatype="<%=at.getValueType().getStringValue()%>"
					enumeration="<%=annotationEnum%>"
					smartenum="<%=at.getDisplayAsList() %>"
					sortable="<%=at.getMultiplicity() == 1%>" 
					filterable="true" 
					exportable="true"
					formatter="<%=formatter%>"
					unit="<%=at.getDefaultUnit()%>"
				/>
				<%
			}
			%>
			<tbl:columndef 
				id="tools"
				title="Tools" 
			/>
			<tbl:columndef 
				id="xt-columns" 
				extensions="<%=columnsInvoker%>" 
				jspcontext="<%=jspContext%>" 
			/>

			<div class="panelgroup">
			<tbl:toolbar
				visible="<%=mode.hasToolbar()%>"
				subclass="<%=root == null ? "bottomborder bg-filled-50" : "topborder leftborder rightborder bg-filled-50" %>"
				>
				<tbl:button 
					id="btnNewRootBioAssaySet"
					disabled="<%=!createPermission%>" 
					image="new.png" 
					title="New root bioassay set&hellip;" 
					tooltip="<%=createPermission ? "Create a new root bioassay set" : "You do not have permission to create bioassay sets"%>" 
					visible="<%=root == null && pluginCount.containsKey(Plugin.MainType.INTENSITY)%>"
				/>
				<tbl:button 
					id="btnDeleteTreeItems"
					disabled="<%=!deletePermission%>" 
					image="delete.png" 
					title="Delete"
					tooltip="<%=deletePermission ? "Delete the selected items" : "You do not have permission to delete items"%>" 
				/>
				<tbl:button 
					id="btnRestoreTreeItems"
					disabled="<%=!writePermission%>" 
					image="restore.png" 
					title="Restore"
					tooltip="<%=writePermission ? "Restore the selected (deleted) items" : "You do not have permission to restore items"%>" 
				/>
				<tbl:button 
					id="btnTreeColumns"
					image="columns.png" 
					title="Columns&hellip;" 
					tooltip="Show, hide and re-order columns" 
				/>
				<tbl:button 
					id="btnTreeImport"
					data-plugin-type="IMPORT"
					image="import.png" 
					title="Import&hellip;" 
					tooltip="Import data" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
				/>
				<tbl:button 
					id="btnTreeExport"
					data-plugin-type="EXPORT"
					image="export.png" 
					title="Export&hellip;" 
					tooltip="Export data" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
				/>
				<tbl:button 
					id="btnTreeRunPlugin"
					data-plugin-type="OTHER"
					image="runplugin.png" 
					title="Run plugin&hellip;" 
					tooltip="Run a plugin" 
					visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
				/>
				<ext:render extensions="<%=toolbarInvoker%>" context="<%=jspContext%>" 
					wrapper="<%=new PrefixSuffixRenderer(jspContext, "<td>", "</td>") %>"/>
			</tbl:toolbar>
			</div>
			<tbl:data style="<%=root == null ? "top: 1.75em;" : "" %>">
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header colspan="3">
						<tbl:presetselector 
							style="border-right: 0px;"
						/>
						</tbl:header>
						<tbl:columnheaders />
					</tbl:headerrow>
					<%
					int numFilters = cc.getNumPropertyFilters();
					int numRows = cc.getFilterRows();
					for (int filterNo = 0; filterNo < numRows; filterNo++)
					{
						boolean lastRow = filterNo == numRows-1;
						%>
						<tbl:headerrow>
							<tbl:header subclass="index" />
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasCheck()%>"
								><base:icon 
									subclass="link table-check"
									image="check_uncheck.png" 
									tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
									visible="<%=lastRow%>"
								/></tbl:header>
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasRadio()%>"
								/>
							<tbl:header 
								subclass="icons" 
								visible="<%=mode.hasIcons()%>"
								>
								<base:icon
									subclass="link table-filter-row-action"
									image="add.png"
									tooltip="Add extra filter row"
									visible="<%=lastRow%>"
								/><base:icon
									subclass="link table-filter-row-action"
									image="remove.png"
									tooltip="Remove this filter row"
									visible="<%=numRows > 1 || numFilters > 0 %>"
									data-remove-row="<%=filterNo%>"
								/>
							</tbl:header>
							<tbl:propertyfilter row="<%=filterNo%>" />
						</tbl:headerrow>
						<%
					}
					%>
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (analysisTree != null)
					{						
						Iterator<Tree.Entry<BasicItem>> baas = analysisTree.entryIterator(root);
						while (baas.hasNext())
						{
							Tree.Entry<BasicItem> entry = baas.next();
							BasicItem item = entry.getNode();
							if (item != null)
							{
								int level = entry.getDepth() - 1;
								int itemId = item.getId();
								String joustId = item.getType().name() + "_" + itemId;
								
								boolean isVisible = true;
								BasicItem parent = entry.getParent().getNode();
								if (parent != null && closed != null)
								{
									String parentId = parent.getType().name()+"_"+parent.getId();
									isVisible = !closed.contains(parentId);
									if (!isVisible) closed.add(joustId);
								}
								String name = "";
								String description = "";
								String tooltip = mode.isSelectionMode() ?
										"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
								boolean removed = false;
								Transformation t = null;
								BioAssaySet bas = null;
								ExtraValue xv = null;
								PluginDefinition plugin = null;
								Job job = null;
								String prefix = "";
								ItemContext ccc = null;
								Item itemType = item.getType();
								if (itemType == Item.TRANSFORMATION)
								{
									t = (Transformation)item;
									removed = t.isRemoved();
									prefix = "T:";
									ccc = tc;
									name = HTML.encodeTags(t.getName());
									description = HTML.encodeTags(t.getDescription());
									try
									{
										job = t.getJob();
										plugin = job.getPluginDefinition();
									}
									catch (Throwable ex)
									{}
								}
								else if (itemType == Item.BIOASSAYSET)
								{
									bas = (BioAssaySet)item;
									removed = bas.isRemoved();
									ccc = cc;
									name = HTML.encodeTags(bas.getName());
									description = HTML.encodeTags(bas.getDescription());
								}
								else if (itemType == Item.EXTRAVALUE)
								{
									xv = (ExtraValue)item;
									prefix = "X:";
									removed = false;
									ccc = xvc;
									try
									{
										ExtraValueType xvType = xv.getExtraValueType();
										name = HTML.encodeTags(xvType.getName());
										description = HTML.encodeTags(xvType.getDescription());
									}
									catch (PermissionDeniedException ex)
									{
										name = xv.getValueType().toString();
										description = "";
									}
									try
									{
										job = xv.getJob();
										plugin = job.getPluginDefinition();
									}
									catch (Throwable ex)
									{}
								}
								
								index++;
								numListed++;
								%>
								<tbl:row
									id="<%="row."+joustId%>"
									style="<%=isVisible ? "" : "display: none;" %>"
									>
									<tbl:header 
										clazz="index"
										><%=index%></tbl:header>
									<tbl:header 
										clazz="check" 
										visible="<%=mode.hasCheck()%>"
										><input 
												type="checkbox" 
												name="<%=prefix+itemId%>" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												<%=ccc.getSelected().contains(itemId) ? "checked" : ""%>
											></tbl:header>
									<tbl:header 
										clazz="check" 
										visible="<%=mode.hasRadio()%>"
										><input 
												type="radio" 
												name="item_id" 
												value="<%=itemId%>" 
												title="<%=name%>" 
												<%=selectedItemId == itemId ? "checked" : ""%>
											></tbl:header>
									<tbl:header 
										clazz="icons" 
										visible="<%=mode.hasIcons()%>"
										><base:icon 
											image="deleted.png"
											id="<%="delete."+itemId %>"
											subclass="<%=deletePermission ? "table-delete-item" : "disabled" %>"
											data-item-type="<%=itemType.name() %>"
											data-item-id="<%=itemId%>"
											data-notify="reloadOnNotify"
											tooltip="This item has been scheduled for deletion" 
											visible="<%=removed%>"
										/>&nbsp;</tbl:header>
									
									<tbl:cell clazz="cell joust" column="name">
										<div id="<%=joustId%>" class="link auto-init"
											data-auto-init="item-link"
											data-item-type="<%=itemType.name() %>" 
											data-item-id="<%=itemId %>"
											data-no-edit="<%=writePermission ? 0 : 1 %>" 
											tabindex="0"
											title="<%=tooltip%>"><%=name%></div>
									</tbl:cell>
									<%
									if (t != null)
									{
										%>
										<tbl:cell column="spots">&nbsp;</tbl:cell>
										<tbl:cell column="reporters">&nbsp;</tbl:cell>
										<tbl:cell column="fileSpots">&nbsp;</tbl:cell>
										<tbl:cell column="fileReporters">&nbsp;</tbl:cell>
										<tbl:cell column="tools" style="white-space: nowrap;">
											<%
											if (createPermission && job != null && plugin != null && plugin.isInteractive() && plugin.hasPermission(Permission.USE))
											{
												// User may not have permission to read plug-in configuration for job
												boolean copyTransformationAllowed = false;
												try
												{
													PluginConfiguration pluginConfig = job.getPluginConfiguration();
													if (pluginConfig == null || pluginConfig.hasPermission(Permission.USE))
													{
														copyTransformationAllowed = true;
													}
												}
												catch (Exception e)
												{}
												if (copyTransformationAllowed)
												{
													%>
													<base:icon 
														subclass="link auto-init"
														data-auto-init="copy-job"
														data-job-id="<%=job.getId() %>"
														image="copy.png" 
														tooltip="Copy this transformation"
													/>
													<%
												}
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />
										</tbl:cell>
										<%
									}
									if (bas != null)
									{
										%>
										<tbl:cell column="spots"><%=bas.getNumSpots()%></tbl:cell>
										<tbl:cell column="reporters"><%=bas.getNumReporters()%></tbl:cell>
										<tbl:cell column="fileSpots"><%=bas.getNumFileSpots()%></tbl:cell>
										<tbl:cell column="fileReporters"><%=bas.getNumFileReporters()%></tbl:cell>
										<tbl:cell column="date">&nbsp;</tbl:cell>
										<tbl:cell column="plugin">&nbsp;</tbl:cell>
										<%
										if (bas.isAnnotated())
										{
											AnnotationSetSnapshot snapshot = manager.getSnapshot(dc, bas.getAnnotationSet().getId());
											for (AnnotationLoaderUtil loader : annotationLoaders)
											{
												%>
												<tbl:cell 
													column="<%="at"+loader.getId()%>"
													><%
													if (loader.find(snapshot, psInfo.reset())) 
													{
														%><tbl:cellvalue 
															list="<%=loader.getValues()%>"
															suffix="<%=loader.getUnitSymbol()%>"
															clazz="<%=psInfo.overridesDefaultAnnotation() ? "ps-annotation" : null%>"
														/><%
													}
													%></tbl:cell>
												<%
											}
										}
										%>
										<tbl:cell column="tools" style="white-space: nowrap;">
											<%
											if (bas.getNumSpots() > 0)
											{
												%>
												<base:icon 
													subclass="link auto-init"
													data-auto-init="bioassayset-plotter"
													data-item-id="<%=itemId %>"
													image="plotter.png" 
													tooltip="A simple plot tool"
												/>
												<base:icon 
													subclass="link auto-init"
													data-auto-init="experiment-explorer"
													data-item-id="<%=itemId %>"
													image="explorer.png" 
													tooltip="Experiment explorer"
												/>
												<%
											}
											%>
											<base:icon 
												subclass="link auto-init"
												data-auto-init="bioassayset-plugin"
												data-plugin-type="EXPORT"
												data-item-id="<%=itemId %>"
												image="export.png" 
												tooltip="Export data"
											/>
											<%
											if (createPermission)
											{
												%>
												<base:icon 
													subclass="link auto-init"
													data-auto-init="bioassayset-plugin"
													data-plugin-type="ANALYZE"
													data-cmd="NewFilteredBioAssaySet"
													data-item-id="<%=itemId %>"
													image="filter.png" 
													tooltip="Create a filtered bioassay set"
												/>
												<base:icon 
													subclass="link auto-init"
													data-auto-init="bioassayset-plugin"
													data-plugin-type="ANALYZE"
													data-item-id="<%=itemId %>"
													image="runplugin.png" 
													tooltip="Run an analysis plugin"
												/>
												<%
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />
										</tbl:cell>
										<%
									}
									if (xv != null)
									{
										%>
										<tbl:cell column="spots"><%=xv.getNumValues()%></tbl:cell>
										<tbl:cell column="reporters">&nbsp;</tbl:cell>
										<tbl:cell column="fileSpots"><%=xv.getNumFileValues()%></tbl:cell>
										<tbl:cell column="fileReporters">&nbsp;</tbl:cell>
										<tbl:cell column="tools" style="white-space: nowrap;">
											<%
											if (createPermission && job != null && plugin != null && plugin.isInteractive() && plugin.hasPermission(Permission.USE))
											{
												// User may not have permission to read plug-in configuration for job
												boolean copyTransformationAllowed = false;
												try
												{
													PluginConfiguration pluginConfig = job.getPluginConfiguration();
													if (pluginConfig == null || pluginConfig.hasPermission(Permission.USE))
													{
														copyTransformationAllowed = true;
													}
												}
												catch (Exception e)
												{}
												if (copyTransformationAllowed)
												{
													%>
													<base:icon 
														subclass="link auto-init"
														data-auto-init="copy-job"
														data-job-id="<%=job.getId() %>"
														image="copy.png"  
														tooltip="Copy this extra value"
													/>
													<%
												}
											}
											%>
											<ext:render extensions="<%=toolsInvoker%>" context="<%=jspContext%>" item="<%=item%>" />
										</tbl:cell>
										<%
									}
									%>
									<tbl:cell column="date"><%=job == null ? "" : dateTimeFormatter.format(job.getEnded())%></tbl:cell>
									<tbl:cell column="plugin"><%=plugin == null ? "" : Base.getLinkedName(ID, plugin, false, true)%></tbl:cell>
									<tbl:cell column="description"><%=description%></tbl:cell>
									<tbl:xt-cells dc="<%=dc%>" item="<%=item%>">
										<tbl:cell column="xt-columns" />
									</tbl:xt-cells>
								</tbl:row>
								<%
							}
						}
					}
					if (numListed == 0)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer note">
							No bioassay sets or transformations were found.
							</div>
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>

		<div id="reloadOnNotify"></div>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
