/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var ItemLists = function()
{
	var lists = {};
	var selectedItemsInParentForm;
	var subtypes;
	
	/**
		Initialize the page.
	*/
	lists.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', lists.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', lists.validateList);
			TabControl.addTabActivateListener('settings.annotations', Annotations.onSwitchToAnnotationsTab);
			
			var frm = document.forms['list'];
			if (frm.source)
			{
				// Create biomaterial list from selection of biomaterials
				var parentForm = Data.get('sourceSelected', 'parent-form');
				selectedItemsInParentForm = window.opener.Table.getSelected(parentForm)
				if (selectedItemsInParentForm.length == 0)
				{
					Doc.element('sourceSelected').disabled = true;
				}
				else
				{
					Forms.checkRadio(frm.source, 'selected');
					Doc.element('lblSelected').innerHTML += ' ['+selectedItemsInParentForm.length+']';
				}
				Events.addEventHandler('sourceSelected', 'click', lists.sourceOnChange);
				Events.addEventHandler('sourcePage', 'click', lists.sourceOnChange);
				Events.addEventHandler('sourceAll', 'click', lists.sourceOnChange);
				lists.sourceOnChange();
			}
			
			// Item subtype
			subtypes = Data.json('page-data', 'subtypes');
			Events.addEventHandler('memberType', 'change', lists.memberTypeOnChange);
			lists.memberTypeOnChange();
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ITEMLIST', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnMergeUnion', lists.mergeOnClick);
			Buttons.addClickHandler('btnMergeIntersection', lists.mergeOnClick);
			Buttons.addClickHandler('btnMergeComplement', lists.mergeOnClick);
			
			Buttons.addClickHandler('btnAddSyncFilter', Buttons.newItem);
			Buttons.addClickHandler('btnRemoveSyncFilter', lists.removeSyncFilterOnClick);
			Buttons.addClickHandler('btnEnable', lists.enableFiltersOnClick);
			Buttons.addClickHandler('btnDisable', lists.disableFiltersOnClick);
			Buttons.addClickHandler('btnResync', lists.resyncFilters);
			// sent after syncing is complete
			Events.addEventHandler('btnResync', 'base-notify', App.reloadWindow);
			
			var inSync = Data.int('sync-data', 'is-in-sync');
			var syncDisabled = Data.int('sync-data', 'sync-disabled');
			if (!syncDisabled)
			{
				if (!inSync)
				{
					Doc.show('not-insync-warning', 'inline');
				}
				else if (Data.int('sync-data', 'num-syncfilters')>0)
				{
					Doc.show('insync-info', 'inline');
				}
			}
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.members', lists.viewMembers);
			if (Doc.element('main.overview'))
			{
				TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			}
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'ITEMLIST'};
			var tableAttributes = {'table-id': 'lists'};
			Buttons.addClickHandler('btnNewItem', lists.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	lists.newItem = function()
	{
		var controller = Items.getController('ITEMLIST');
		Table.submitToPopup('lists', 'NewItem', controller.width, controller.height);
	}
	
	lists.viewMembers = function()
	{
		var listId = Data.get('page-data', 'item-id');
		var url = 'members/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&list_id='+listId;
		location.replace(url);
	}
	
	lists.mergeOnClick = function(event)
	{
		var target = event.currentTarget;
		var mergeType = Data.get(target, 'merge-type');
		var listId = Data.get('page-data', 'item-id');
		
		var url = 'merge.jsp?ID='+App.getSessionId();
		url += '&item_id='+listId;
		url += '&mergeType='+mergeType;
		
		Dialogs.openPopup(url, 'Merge', 600, 400);
	}

	lists.removeSyncFilterOnClick = function()
	{
		var tableDiv = Doc.element('tbl.syncFilters');
		var numSelected = Table.checkIfSelected(tableDiv);
		if (numSelected == 0) return;
		
		if (!confirm('You are about to remove '+numSelected+' filters. Continue?'))
		{
			return;
		}
		Table.submitToPopup(tableDiv, 'DeleteItems', 300, 200);
	}
	
	lists.enableFiltersOnClick = function()
	{
		var tableDiv = Doc.element('tbl.syncFilters');
		var numSelected = Table.checkIfSelected(tableDiv);
		if (numSelected == 0) return;
		
		Table.submitToPopup(tableDiv, 'EnableFilters', 300, 200);
	}

	lists.disableFiltersOnClick = function()
	{
		var tableDiv = Doc.element('tbl.syncFilters');
		var numSelected = Table.checkIfSelected(tableDiv);
		if (numSelected == 0) return;

		Table.submitToPopup(tableDiv, 'DisableFilters', 300, 200);
	}
	
	lists.resyncFilters = function()
	{
		var listId = Data.get('page-data', 'item-id');
		var url = 'resync.jsp?ID='+App.getSessionId();
		url += '&item_id='+listId;
		Dialogs.openPopup(url, 'ReSync', 600, 400);
	}
	
	lists.validateList = function()
	{
		var frm = document.forms['list'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	// Submit the form
	lists.save = function()
	{
		var frm = document.forms['list'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			if (selectedItemsInParentForm)
			{
				Forms.addHidden(frm, 'items', selectedItemsInParentForm.join(','));
			}
			frm.memberType.disabled = false;
			frm.submit();
		}
	}
	
	lists.memberTypeOnChange = function()
	{
		var frm = document.forms['list'];
		var memberType = frm.memberType.value;
		
		if (memberType == 'RAWBIOASSAY')
		{
			Doc.show('rawDataTypeRow');
			Doc.hide('subtypeRow');
		}
		else
		{
			Doc.hide('rawDataTypeRow');
			Doc.show('subtypeRow');
			
			frm.subtype_id.length = 0;
			frm.subtype_id[0] = new Option('- any -', 0);
			for (var i = 0; i < subtypes.length; i++)
			{
				var subtype = subtypes[i];
				if (subtype.itemType == memberType)
				{
					var opt = new Option(subtype.name, subtype.id, false, subtype.current);
					if (subtype.description) opt.title = subtype.description;
					frm.subtype_id[frm.subtype_id.length] = opt;
				}
			}
		}
	}
	
	lists.sourceOnChange = function()
	{
		var frm = document.forms['list'];
		frm.syncFilter.disabled = !Doc.element('sourceAll').checked;
	}
	
	lists.initElements = function(element, autoInit)
	{
		if (autoInit == 'sync-filter-edit') 
		{
			Events.addEventHandler(element, 'click', lists.modifySyncFilter);
			Events.addEventHandler(element, 'base-selected-start', lists.syncFilterModified);
		}
		else if (autoInit == 'sync-filter-view')
		{
			Events.addEventHandler(element, 'click', lists.viewSyncFilterInTable);
		}
		else if (autoInit == 'clone')
		{
			Buttons.addClickHandler(element, lists.copyListOnClick);
		}
	}
	
	lists.modifySyncFilter = function(event)
	{
		var target = event.currentTarget;
		var syncFilterId = Data.get(target, 'syncfilter-id');
		var sourceItemType = Data.get(target, 'source-item-type');
		
		var url = 'syncfilter/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=CopySyncFilterToItemContext';
		url += '&syncfilter_id='+syncFilterId;

		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, 
			function(request) 
			{
				Dialogs.selectItem(sourceItemType, target, 'selectfilter');
			});
		request.send(null);
	}
	
	// When we are sure the current filter is in the item context,
	// open the dialog
	lists.syncFilterModified = function(event)
	{
		var syncFilterId = Data.get(event.currentTarget, 'syncfilter-id');
		var url = 'syncfilter/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=CopyItemContextToSyncFilter';
		url += '&syncfilter_id='+syncFilterId;
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, 
			function(request)
			{
				location.reload();
			});
		request.send(null);
	}

	lists.viewSyncFilterInTable = function(event)
	{
		var target = event.currentTarget;
		var syncFilterId = Data.get(target, 'syncfilter-id');
		var sourceItemType = Data.get(target, 'source-item-type');
		
		var url = 'syncfilter/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=CopySyncFilterToItemContext';
		url += '&syncfilter_id='+syncFilterId;

		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, 
			function(request) 
			{
				Items.listWithHistory(sourceItemType);
			});
		request.send(null);
	}

	
	// Click handler for cloning a list
	lists.copyListOnClick = function(event)
	{
		var target = event.currentTarget;
		var listId = Data.get(target, 'item-id');
		Items.newItem('ITEMLIST', '&clone_id='+listId);
	}

	
	return lists;
}();

Doc.onLoad(ItemLists.initPage);
Doc.addElementInitializer(ItemLists.initElements);
