<%-- $Id: view_session.jsp 7414 2017-10-12 12:07:09Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Session"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.UserDevice"
	import="net.sf.basedb.core.Client"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.GeoLocation"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="java.util.Date"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
	private static final Item itemType = Item.SESSION;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String tab = Values.getString(request.getParameter("tab"), "properties");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	Session s = Session.getById(dc, itemId);
	
	boolean readUser = true;
	User user = null;
	try
	{
		user = s.getUser();
	}
	catch (Exception ex)
	{
		readUser = false;
	}
	boolean readClient = true;
	Client client = null;
	try
	{
		client = s.getClient();
	}
	catch (Exception ex)
	{
		readClient = false;
	}
	boolean readDevice = true;
	UserDevice device = null;
	try
	{
		device = s.getDevice();
	}
	catch (Exception ex)
	{
		readDevice = false;
	}
	final String mapURL = GeoLocation.getURLToMap(s.getLocationLatitude(), s.getLocationLongitude());
	String title = "View session";
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	%>
	<base:page type="popup" title="<%=title%>" id="view-page">
	<base:head scripts="~sessions.js" />
	<base:body data-read-only="1">
		
		<h1><%=title%> <base:help helpid="session.view" /></h1>
		<div id="page-data" data-item-id="<%=itemId%>"></div>
		
		<div class="content">
			<table class="fullform larger bottomborder outlined">
			<tr>
				<th>User</th>
				<td><%=Base.getEncodedName(user, readUser)%></td>
			</tr>
			<tr>
				<th>Client application</th>
				<td><%=Base.getEncodedName(client, !readClient)%></td>
			</tr>
			<tr>
				<th>Device</th>
				<td><%=Base.getEncodedName(device, !readDevice)%></td>
			</tr>
			<tr>
				<th>Login time</th>
				<td><%=dateTimeFormatter.format(s.getLoginTime())%></td>
			</tr>
			<tr>
				<th>Logout time</th>
				<td><%=dateTimeFormatter.format(s.getLogoutTime())%></td>
			</tr>
			<tr>
				<th>Authentication method</th>
				<td><%=HTML.encodeTags(s.getAuthenticationMethod())%></td>
			</tr>
			<tr>
				<th>Remote ID</th>
				<td><%=HTML.encodeTags(s.getRemoteId())%></td>
			</tr>
			<tr>
				<th>Location</th>
				<td><%=HTML.encodeTags(s.getLocation())%>
					<base:icon href="<%=mapURL%>" image="map.png" tooltip="Show map" visible="<%=mapURL != null %>" />
				</td>
			</tr>
			<tr>
				<th>Impersonated</th>
				<td><%=s.getImpersonated()%></td>
			</tr>
			<tr class="dynamic">
				<th>Comment</th>
				<td><%=HTML.niceFormat(s.getLoginComment())%></td>
			</tr>
			</table>
		</div>
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="close" title="Close" />
		</base:buttongroup>

	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.commit();
}

%>
